﻿/*
   Copyright 2019 Nils Kopal, nils.kopal<at>cryptool.org
                  Christian Bender, christian.bender<at>cryptool.org

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

using MysteryCipher.Ciphers;
using System;
using System.IO;

namespace FileEncrypter
{
    public class Program
    {
        /// <summary>
        /// FileEncrypter -- Mystery Cipher 1 -- Version 1
        /// (C) Nils Kopal, Christian Bender, 2019
        /// Command line parameters:
        /// -e Encrypt mode
        /// -d Decrypt mode
        /// -k Round keys(four little endian 16-bit unsigned integer values)
        /// -i Input file
        /// -o Output file
        /// -h Show help
        ///
        /// Example encryption: FileEncrypter.exe -e -k 100 200 300 400 -i input.bin -o output.bin
        /// The call above encryps the input.bin using the roundkeys (100, 200, 300, and 400) and writes the result to output.bin
        /// Warning: Output files are overwritten
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            if(args.Length == 0)
            {
                ShowHelp();
                return;
            }
            foreach (var str in args)
            {
                if (str.ToLower().Trim().Equals("-h"))
                {
                    ShowHelp();
                    return;
                }
            }

            //Search for encrypt or decrypt
            Mode mode = Mode.None;
            foreach(var str in args)
            {
                if (str.ToLower().Trim().Equals("-e"))
                {
                    mode = Mode.Encrypt;
                    break;
                }
                if (str.ToLower().Trim().Equals("-d"))
                {
                    mode = Mode.Decrypt;
                    break;
                }
            }
            if(mode == Mode.None)
            {
                Console.WriteLine("No -e or -d parameter given");
                return;
            }

            bool keyFound = false;
            int[] roundkeys = new int[4];
            //Search for 4 round keys
            for(int i=0; i < args.Length; i++)
            {
                string str = args[i];
                if (str.ToLower().Trim().Equals("-k"))
                {
                    keyFound = true;
                    //now, get 4 16bit int values
                    if(i + 4 > args.Length - 1)
                    {
                        Console.WriteLine("No four valid round keys (four little endian 16-bit unsigned integer values) given");
                        return;
                    }
                    for(int j = i + 1; j < i + 1 + 4; j++)
                    {
                        str = args[j];
                        try
                        {
                            roundkeys[j - i - 1] = UInt16.Parse(str);
                        }
                        catch (Exception)
                        {
                            Console.WriteLine("\"{0}\" is not a valid little endian 16-bit unsigned integer value roundkey", str);
                            return;
                        }
                    }
                }
            }
            if (!keyFound)
            {
                Console.WriteLine("No -k paramater for roundkeys given");
            }

            //Search for input file
            string inputfile = null;
            string outputfile = null;
            for (int i = 0; i < args.Length; i++)
            {
                string str = args[i];
                if (str.ToLower().Trim().Equals("-i"))
                {
                    try
                    {
                        inputfile = args[i + 1];
                    }
                    catch(Exception)
                    {
                        break;
                    }
                    
                }
                if (str.ToLower().Trim().Equals("-o"))
                {
                    try
                    {
                        outputfile = args[i + 1];
                    }
                    catch (Exception)
                    {
                        break;
                    }

                }
            }
            if(inputfile == null)
            {
                Console.WriteLine("No valid input file with -i defined");
                return;
            }
            if (outputfile == null)
            {
                Console.WriteLine("No valid output file with -i defined");
                return;
            }

            string keys = roundkeys[0] + ", " + roundkeys[1] + ", " + roundkeys[2] + ", and " + roundkeys[3];
            Console.WriteLine("{0} file {1} with round keys {2} and writing the result to {3}",
                mode == Mode.Encrypt ? "Encrypting" : "Decrypting",
                inputfile,
                keys,
                outputfile);
            try
            {
                MysteryCipher1 cipher = new MysteryCipher1();
                byte[] input = File.ReadAllBytes(inputfile);
                byte[] output;

                if (mode == Mode.Encrypt)
                {
                    output = cipher.Encrypt(input, roundkeys);
                }
                else
                {
                    output = cipher.Decrypt(input, roundkeys);
                }
                File.WriteAllBytes(outputfile, output);
                Console.WriteLine("Successfully written {0} bytes", output.Length);
            }
            catch(Exception ex)
            {
                Console.WriteLine("Exception {0} occured during execution", ex.Message);
            }
        }

        /// <summary>
        /// Shows the help output in the console
        /// </summary>
        public static void ShowHelp()
        {
            Console.WriteLine("FileEncrypter -- Mystery Cipher 1 -- Version 1");
            Console.WriteLine("(C) Nils Kopal, Christian Bender, 2019");
            Console.WriteLine("Command line parameters:");
            Console.WriteLine(" -e          Encrypt mode");
            Console.WriteLine(" -d          Decrypt mode");
            Console.WriteLine(" -k          Round keys (four little endian 16-bit unsigned integer values)");
            Console.WriteLine(" -i          Input file");
            Console.WriteLine(" -o          Output file");
            Console.WriteLine(" -h          Show help");
            Console.WriteLine();
            Console.WriteLine("Example encryption: FileEncrypter.exe -e -k 100 200 300 400 -i input.bin -o output.bin" );
            Console.WriteLine("The call above encryps the input.bin using the roundkeys (100, 200, 300, and 400) and writes the result to output.bin");
            Console.WriteLine("Warning: Output files are overwritten");
        }

        enum Mode
        {
            None,
            Encrypt,
            Decrypt
        }
    }
}
